from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFTypes.ReferenceEngine import ReferenceEngine
from Products.CMFTypes.debug import log, log_exc
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import time


class ContentTool(UniqueObject, ActionProviderBase, SimpleItem, ReferenceEngine):
    """ Install unique Ids on BaseContent derived objects """
    id        = 'content_tool'
    meta_type = 'CMFTypes Content Tool'


    manage_options=(
        SimpleItem.manage_options +
        ({  'label'  : 'View',
            'action' : 'view',
            },        
         {  'label'  : 'ReIndex',
            'action' : 'index',
            },
         )
        )

        
    def getObject(self, uid):
        object = None
        catalog = getToolByName(self, 'portal_catalog')
        result  = catalog({'UID' : uid})
        if result:
            object = result[0].getObject()

        return object

    def enum(self, callback, *args, **kwargs):
        catalog = getToolByName(self, 'portal_catalog')
        keys = catalog.uniqueValuesFor('UID')
        for uid in keys:
            o = self.getObject(uid)
            if o:
                callback(o, *args, **kwargs)
            else:
                log("no object for %s" % uid)

    def _genId(self, object):
        catalog = getToolByName(self, 'portal_catalog')
        keys = catalog.uniqueValuesFor('UID')

        cid = object.getId()
        i = 0
        while cid in keys:
            cid = "%s-%s" % (object.getId(), i)
            i = int((time.time() % 1.0) * 10000)

        return cid

    def registerContent(self, object):
        """register a content object and set its unique id"""
        cid = self.getUidFrom(object)
        if cid is None:
            cid = self._genId(object)
            self.setUidOn(object, cid)

        return cid

    def unregisterContent(self, object):
        """remove all refs/backrefs from an object"""
        cid = self.getUidFrom(object)
        self._delReferences(cid)
        return cid
            

    def getUidFrom(self, object):
        """return the UID for an object or None"""
        value = None

        if hasattr(object, "_getUID"):
            value = object._getUID()

        return value

    def setUidOn(self, object, cid):
        if hasattr(object, "_setUID"):
            object._setUID(cid)
            
        
    def Content(self):
        """Return a list of all the content ids"""
        catalog = getToolByName(self, 'portal_catalog')
        keys = catalog.uniqueValuesFor('UID')
        result = []
        for key in keys:
            result.append((key, self.getObject(key)))
        result.sort()

        return result

    view = PageTemplateFile('www/viewContents', globals())
    
    
    def index(self):
        """assign UIDs to all basecontent objects"""

        def _index(object, content_tool):
            from BaseContent import BaseContent
            if isinstance(object, BaseContent):
                content_tool.registerContent(object)

        self.enum(_index, self)

        return "Sucessfully assigned UIDs"


    
InitializeClass(ContentTool)

    
